@@ -139,6 +139,8 @@ int bt_open_port(char* port)
 int bt_close_port(int final) 
 {
 	int retval = 0;
+	
+	logger("INF", "bt_close_port");
 
 	if (!portfd) { /* already closed */
 		if (final) logger("INF", "Already closed ?");
@@ -166,7 +168,7 @@ int bt_read(char* buf, int len)
 	return read(portfd, buf, len);
 }
 
-int bt_read_tmo(char* buf, int len, int timeout) 
+static int bt_read_tmo(char* buf, int len, int timeout) 
 {
 	int ok;
 	fd_set rdfds;
@@ -220,7 +222,11 @@ int  bt_readchar(char* c, int timeout)
 	*c = 0;
         
 	int rv =  bt_read_tmo(buf, 1, timeout);
-
+	/*if (rv != 0) {
+		sprintf(tmp, "bt_read_tmo returns: %d", rv);
+		logger("DBG", tmp);
+	}*/
+	
 	if (rv == 1) {
         	*c =  buf[0];
         	return 1;
@@ -775,7 +781,7 @@ int openSocketPort(int type, int port, char *path)
 
 int closeSocketPort(int type, int final) 
 {
-	if (final) logger("INF", "closeSocketPort");
+	/*if (final)*/ logger("INF", "closeSocketPort");
 	
 	if (portfd) { 
 		close(portfd);
@@ -827,7 +833,7 @@ int listenAndAcceptSocketConn(int type)
 
 	while (stillRun) {
 	
-	        //logger("INF", "listenAndAcceptSocketConn: accept");
+	        logger("INF", "listenAndAcceptSocketConn: accept");
 			       
 		listen(sportfd,5);
 		 
