@@ -22,6 +22,7 @@
 #define _GNU_SOURCE
 #include <stdio.h>
 
+#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -565,7 +566,7 @@ static char* substParams(const char* in, cmdParams *params) {
 
 static int execSimpleCmd(const char *cmd, int toFile)
 {
-	if (cmd == NULL || remoteOn != 1) {
+	if (cmd == NULL || strlen(cmd) == 0 || remoteOn != 1) {
         	logger("DBG", "execSimpleCmd null input or remote if OFF");
         	return EXIT_OK;
         }
@@ -588,7 +589,13 @@ static int execSimpleCmd(const char *cmd, int toFile)
         	strcat(p, " > ");		
                 strcat(p, getResFile());
         } else {
-        	if (!index(p, '&')) {
+	        // How to handle & inside file names ?
+        	//if (!index(p, '&') ) {
+		
+		int idx = strlen(cmd)-1;
+		while (idx > 0 && isspace(cmd[idx])) { idx--; }
+		
+		if (cmd[idx] != '&') {
                 	strcat(p, "&");
                 }
         }
