@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009,2010 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ static char* dupVarValue(char *name, int size)
 {       
         varData* v = searchVar(name, size);
 	if (v != NULL && v->value != NULL && v->value[0] != '\0') {
-		char *d = malloc(v->size+1);
+		char *d = calloc(1,v->size+1);
 		strncpy(d,v->value,v->size);
 		d[v->size] = '\0';
 		return d;
@@ -82,7 +82,9 @@ char* getDevice()
 
 int getBaudrate()
 {
-	char br[16] = "";	// enough for 234000 
+	char br[16];	// enough for 234000 
+	memset(br,0,16);
+	
         varData* v = searchVar("Baudrate", 12);
         if (v != NULL) {
 	        strncpy(br,v->value,v->size);
@@ -115,7 +117,8 @@ int getBaudrate()
 
 int getRetrySecs()
 {
-	char rs[16] = "";	// should be enough 
+	char rs[16];	// should be enough 
+	memset(rs,0,16);
         varData* v = searchVar("RetrySeconds", 12);
         if (v != NULL) {
 	        strncpy(rs,v->value,v->size);
@@ -126,22 +129,44 @@ int getRetrySecs()
 
 int autoConnect()
 {
-        varData* v = searchVar("AutoConnect", 11);
+        varData* v = searchVar(AUTOCONN_STR, 11);
         if (v != NULL && strncmp(v->value,"true",4) == 0) {
 		return 1;
 	}
 	return 0;
 }
 
-int getAutoRepeat()
+int getIViewer()
 {
-        varData* v = searchVar("AutoRepeat", 10);
+        varData* v = searchVar("IViewer", 7);
         if (v != NULL && strncmp(v->value,"true",4) == 0) {
 		return 1;
 	}
 	return 0;
 }
 
+
+int getAutoRepeat()
+{
+        varData* v = searchVar(AUTOREPEAT_STR, 10);
+        if (v != NULL && strncmp(v->value,"true",4) == 0) {
+		return 1;
+	}
+	return 0;
+}
+
+static type_key* repeatCMD = NULL;
+
+void setRepeatNow(type_key* repeat) 
+{
+	repeatCMD = repeat;
+}
+
+type_key* repeatNow()
+{
+	return repeatCMD;
+}
+
 int getUseScreen()
 {
         varData* v = searchVar("TwoWayComm", 10);
@@ -153,7 +178,7 @@ int getUseScreen()
 
 char* getCharset()
 {
-        return dupVarValue("CharSet", 7);
+        return dupVarValue(CHARSET_STR, 7);
 }
 
 char* getToMainMenu()
@@ -386,9 +411,9 @@ static type_key* findItemInMode(mode *mode, char *key, int *flag, cmdParams *par
 	if (start != NULL && finish != NULL && start != key) {        // Do not match "(...)=..."
 		It = mode->keys;
 	
-		char tag  [MAXARGLEN];
-		char index[6];
-		char value[MAXARGLEN];
+		char tag  [MAXARGLEN];memset(tag,  0,MAXARGLEN);
+		char index[6];        memset(index,0,6);
+		char value[MAXARGLEN];memset(value,0,MAXARGLEN);
 	
 		strncpy(tag, key, start-key);
 		*(tag+(start-key)) = '\0';
@@ -486,6 +511,7 @@ type_key* findItem(char *keyIn, int *flag, cmdParams *params)
         }
 
 	char key[MAXARGLEN];
+	memset(key,0,MAXARGLEN);
 
 	if (*keyIn == '\0') {
 		logger("INF","Got empty key");   
