@@ -262,8 +262,30 @@ void closePort(int final)
 	}
 }
 
+static void handleActiveCall(void) 
+{
+	if (callTimer >= 20) {
+		logger("DBG", "[DS]: Get call list again");
+
+		// Will set global callerId [MAXLEN];
+		int ret = getClip(callerId);
+		if (ret == EXIT_EXACT) {
+			DEBUG2("[DS]: Call still active (%s)", callerId);
+		} else {
+			logger("DBG","[DS]: Call was finished");
+			callOn = 0;
+			strcpy(callerId, "NO CALLER ID");
+
+			sendEventToExecutor(ID_EVT_ENDCALL);
+		}
+		callTimer = 0;
+	} else {
+		callTimer++;
+	}
+}
+
 //
-// retuencodeis used in case of exit only
+// return code is used in case of exit only
 //
 static int proceedDisconnect(void) 
 {
@@ -659,7 +681,7 @@ gpointer dispatcherRoutine(gpointer thread)
 {
 	char answer[MAXCMDLEN];
 	int ret = EXIT_OK;
-	int retRead;
+	int retRead = 0;
 
 	logger("DBG","[DS]: start dispatcher thread");
 
@@ -686,6 +708,7 @@ gpointer dispatcherRoutine(gpointer thread)
 		char* dv = getDevice();
 		
 		logger("DBG","[DS]: ************ outer loop **********");
+		//printf("opening server socket ...\n");
 		
 		// try to open port
 		if(openPort(dv) == 1) {   // Open device
@@ -706,29 +729,39 @@ gpointer dispatcherRoutine(gpointer thread)
 		    
 		    if (initPort() == 1 ) {  // Init modem: AT, ATE0, AT+CMER, in server mode waits for connection
 
- 		       dispIsJoinable = TRUE;
+ 		        dispIsJoinable = TRUE;
 		       
-                       isConnected = 1;
+                        isConnected = 1;
                         
 			if (usePassword()) {
-				int PassOk = 0;
-				
+			
 				char *pass = getStoredPass();
-				
 				if (pass) {
-				
-					sendToFrontEnd("Verifying password");
-					
-					ret = writeSocketConn("Get(password);", 14);
-                			if (ret != EXIT_OK) {
-                				logger("DBG","[DS]: Fails in writeSocketConn()");
-	    					continue;
-                			}
-					
+				        int gotReply = 1;
 					while (1) {
+					        
+						if (gotReply == 1) {
+							sendToFrontEnd("Verifying password");
+					
+							ret = writeSocketConn("Get(password);", 14);
+                					if (ret != EXIT_OK) {
+                						logger("DBG","[DS]: Fails in writeSocketConn()");
+	    							continue;
+                					}
+							if (getHttp()) {
+								ret = writeSocketConn("End();", 6);
+                						if (ret != EXIT_OK) {
+                							logger("DBG","[DS]: Fails in writeSocketConn()");
+	    								continue;
+                						}
+							}
+							gotReply = 0;
+						}
+					
 						retRead = read_command(answer, sizeof(answer));
 				
 						if (retRead == EOF) {
+						        logger("DBG","[DS]: Got disconnect on verifying password");
 							sendToFrontEnd("Disconnected");
 							sendDisconnect();
 							break;
@@ -736,6 +769,8 @@ gpointer dispatcherRoutine(gpointer thread)
 							DEBUG2("[DS]: Password => %s\n",answer);
 							DEBUG2("[DS]: Password => %s\n",answer+16);
 							DEBUG2("[DS]: Compare to => %s\n",pass);
+							
+							gotReply = 1;
 				
 							if (memcmp(answer,   DEF_MSG, 4) == 0 && 
 				    	                    memcmp(answer+4, "_PASSWORD_(,", 12) == 0) {	    // Got verification response from Java client
@@ -743,7 +778,8 @@ gpointer dispatcherRoutine(gpointer thread)
 								int plen = strlen(pass);
 								if (memcmp(answer+16, pass, plen) == 0) {	// _PASSWORD_(,<password here>) 
 									logger("INF","[DS]: Password OK");
-									PassOk = 1;
+									free(pass);
+									break;
 								} else {
 									logger("INF","[DS]: Verification failed. Incorrect password.");
 									sendToFrontEnd("Verification failed");
@@ -753,147 +789,119 @@ gpointer dispatcherRoutine(gpointer thread)
 								logger("INF","[DS]: Verification failed. Incorrect response.");
 								sendToFrontEnd("Verification failed");
 							}
-							break;
 						}
 						
 						if (checkMsgQueue() == EXIT_ABORT) {
 							free(dv);
 							return NULL;
 						}
-
 						usleep(50000);
 					}
 				} else {
 					logger("DBG","[DS]: No password specified. Skip check.");
 				}
-				
-				free(pass);
-				if (PassOk == 0) {
-				        writeSocketConn("Set(title,PASSWORD INVALID);", 28);
-					
-					sleep(1);
-					
-					closePort(1);
-					isConnected = 0;
-					
-					sleep(5);
-					continue;	
-				}
 			}
 			
-			sendToFrontEnd("Connected");
-			sendEventToExecutor(ID_EVT_CONNECT);
+			if (retRead != EOF) {
+				sendToFrontEnd("Connected");
+				sendEventToExecutor(ID_EVT_CONNECT);
 		        
-			logger("INF","[DS]: Start event forwarding");
+				logger("INF","[DS]: Start event forwarding");
 		
-			while (1) {	 
-		    		//logger("DBG","[DS]: main loop");
+				while (1) {	 
+		    			//logger("DBG","[DS]: main loop");
 		    
-		    		retRead = read_command(answer, sizeof(answer));
-				//printf("GOT %d\n",retRead);
+		    			retRead = read_command(answer, sizeof(answer));
+					//if (retRead != 0) printf("GOT %d\n",retRead);
 				
-		    		if (retRead == EOF) {
-	    				logger("INF", "[DS]: Got disconnected");
-					printf("Got disconnected\n");
+		    			if (retRead == EOF) {
+	    					logger("INF", "[DS]: Got disconnected");
+						//printf("Got disconnected\n");
 					
-					freeBtAddress();
+						freeBtAddress();
 					
-					if (gotExitSignal) {
-			   	 		logger("INF", "[DS]: Got signal, exiting");
-						closePort(1);
+						if (gotExitSignal) {
+			   	 			logger("INF", "[DS]: Got signal, exiting");
+							closePort(1);
+							free(dv);
+							return NULL;
+						
+						}
+						sendToFrontEnd("Disconnected");
+						sendDisconnect();
+						sendEventToExecutor(ID_EVT_DISCONNECT);
+						    		    
+	    					if (serverMode == CLIENT_AT    || 
+					    	    serverMode == CLIENT_ILIRC || 
+					            serverMode == CLIENT_NOAT) {  // In server mode we will reuse already opened socket
+					    
+	    		    				logger("INF", "[DS]: Closing the port");
+			    				closePort(1);
+						}   
+	    					ret = EXIT_NOK;
+                                        
+                                        	isConnected = 0;
+	    		
+	    					break;
+					
+		    			} else if (retRead > 0) {
+	    	    
+		    				if (serverMode == CLIENT_NOAT) {	// in IR communication a lot of 'empty' chars (= -1) could be added
+							char * k2 = answer;
+							while (*k2 != '\0') {
+								if (*k2 == -1) {
+									*k2 = '\r';
+								}
+								k2++;
+							}
+						}
+
+						parseCommand(answer);
+						if (getHttp()) {
+					
+					 		eMessage* em = malloc(sizeof(eMessage));
+							em->type  = EM_AS_IS;
+							em->value = strdup("End();");
+							sendToExecutor(em);
+						}
+	    	    			}
+		    
+		    			// Is call still active (timeout about 1 seconds) ?
+		    			if (callOn) {
+					        handleActiveCall();
+	    	    			}
+	    	    		
+					ret = checkMsgQueue();
+					if (ret == EXIT_ABORT) {
 						free(dv);
 						return NULL;
-						
+					} else if (ret == EXIT_DISCON) {
+						break;
 					}
-					sendToFrontEnd("Disconnected");
-					sendDisconnect();
-					sendEventToExecutor(ID_EVT_DISCONNECT);
-						    		    
-	    				if (serverMode == CLIENT_AT    || 
-					    serverMode == CLIENT_ILIRC || 
-					    serverMode == CLIENT_NOAT) {  // In server mode we will reuse already opened socket
-					    
-	    		    			logger("INF", "[DS]: Closing the port");
-			    			closePort(1);
-					}   
-	    				ret = EXIT_NOK;
-                                        
-                                        isConnected = 0;
-	    		
-	    				break;
-		    		} else if (retRead > 0) {
-	    	    
-		    			if (serverMode == CLIENT_NOAT) {	// in IR communication a lot of 'empty' chars (= -1) could be added
-						char * k2 = answer;
-						while (*k2 != '\0') {
-							if (*k2 == -1) {
-								*k2 = '\r';
-							}
-							k2++;
-						}
-					}
-
-					parseCommand(answer);
-					if (getHttp()) {
-					
-					 	eMessage* em = malloc(sizeof(eMessage));
-						em->type  = EM_AS_IS;
-						em->value = strdup("End();");
-						sendToExecutor(em);
-					}
-	    	    		}
-		    
-		    		// Is call still active (timeout about 1 seconds) ?
-		    		if (callOn) {
-	    				if (callTimer >= 20) {
-	    		   			 logger("DBG", "[DS]: Get call list again");
-	    		    
-	    		    			// Will set global callerId     [MAXLEN];
-	    		    			ret = getClip(callerId);
-	    		    			if (ret == EXIT_EXACT) {
-	    						DEBUG2("[DS]: Call still active (%s)", callerId);
-	    		    			} else {
-	    						logger("DBG","[DS]: Call was finished");
-	    						callOn = 0;
-	    						strcpy(callerId, "NO CALLER ID");
-
-							sendEventToExecutor(ID_EVT_ENDCALL);
-	    		    			}
-
-	    		   			callTimer = 0;
-	    				} else {
-	    		    			callTimer++;
-	    				}
-	    	    		}
-	    	    		
-				ret = checkMsgQueue();
-				if (ret == EXIT_ABORT) {
-					free(dv);
-					return NULL;
-				} else if (ret == EXIT_DISCON) {
-					break;
-				}
 				
-	    	    		// Main loop timer (1/50 of second)
-		    		usleep(20000);
-		 	}  
-		    }	// init port
-		}	// open port
+	    	    			// Main loop timer (1/50 of second)
+		    			usleep(20000);
+		 		}
+			}   	// not EOF after password verification
+		    }		// init port
+		}		// open port
 		
 		free(dv);
 		
-		printf("Connection closed or lost\n");
+		//printf("Connection closed or lost\n");
+		logger("INF", "[DS]: Connection closed or lost");
 
 		// Can't open port or it closed again
-		if (dmn || 
-		    serverMode == SERVER_BT   || 
-		    #ifdef USE_L2CAP 
-		    serverMode == SERVER_L2CAP || 
-		    #endif
-		    serverMode == SERVER_TCP  ||
-		    serverMode == SERVER_UX   || 
-		    serverMode == CLIENT_NOAT ||
-		    ret == EXIT_DISCON) {
+		if (!gotExitSignal &&
+		    (dmn || 
+		     serverMode == SERVER_BT   || 
+		     #ifdef USE_L2CAP 
+		     serverMode == SERVER_L2CAP || 
+		     #endif
+		     serverMode == SERVER_TCP  ||
+		     serverMode == SERVER_UX   || 
+		     serverMode == CLIENT_NOAT ||
+		     ret == EXIT_DISCON)) {
 		    
 	    		int timeout;
 	    	
@@ -905,7 +913,7 @@ gpointer dispatcherRoutine(gpointer thread)
 		            serverMode == SERVER_UX  || 
 		            serverMode == CLIENT_NOAT) {
 			    
-				timeout = 5;	// wait only 5 seconds  
+				timeout = 2;	// wait only 2 seconds  
 				 
 			} else if (ret == EXIT_DISCON) {
 			
@@ -917,7 +925,7 @@ gpointer dispatcherRoutine(gpointer thread)
 	    		}
 
 	    		INFO2("[DS]: Wait %d seconds to connect/open server socket ...", timeout);
-			printf("Wait %d seconds to connect/open server socket ...\n", timeout);
+			//printf("Wait %d seconds to connect/open server socket ...\n", timeout);
 			
 			dispIsJoinable = FALSE;
 	    		sleep(timeout);
