@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006,2007,2008,2009 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006,2007,2008,2009,2010 Mikhail Fedotov <anyremote@mail.ru>
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@ extern GAsyncQueue* q2main;
 
 char logfile [MAXLEN];
 static char tofile[MAXLEN];
+static char cmdfile[MAXLEN];
 
 timerCmd* timers  = NULL;
 varData*  vars    = NULL;
@@ -157,9 +158,10 @@ void printTime()
         timestr[strlen(timestr)-1]=0;
 	logger("CFG", timestr);
 }	
-void setResFile()
+void setCmdFiles()
 {
-	initFile(tofile, TOFILE);
+	initFile(tofile,  TOFILE);
+	initFile(cmdfile, CMDFILE);
 }
 
 char* getResFile()
@@ -167,6 +169,11 @@ char* getResFile()
 	return tofile;
 }
 
+char* getCmdFile()
+{
+	return cmdfile;
+}
+
 //////////////////////////////////////////////////////////////////////////////////
 //
 // Functions related to timer commands
@@ -216,18 +223,38 @@ timerCmd* createTimer(cmdItem *ci)
         	tm->next = timers;
         }
         timers = tm;
+	
+        DEBUG2("createTimer(): timer created %s %d",tm->timer->descr,tm->timer->tdata->timeout);
 
         return tm;
 }
 
 void cancelTimer(char *id) 
 {
-	DEBUG2("cancelTimer() >%s<", id);
+	DEBUG2("cancelTimer() >%s<", id?id:"NULL");
 
         timerCmd *prev = NULL;
         timerCmd *tm = findTimer(id, (timerCmd **) &prev);
         if (tm == NULL) {
-        	logger("DBG", "Cant find timer to cancel!");
+        	logger("DBG", "Can not find timer to cancel!");
+        	return ;
+        }
+	
+	// postpone removal because in general timer can be canceled
+	// inside itself
+        tm->status    = TIMER_CANCEL;
+
+        return;
+}
+
+static void deleteTimer(char *id) 
+{
+	DEBUG2("deleteTimer() >%s<", id?id:"NULL");
+
+        timerCmd *prev = NULL;
+        timerCmd *tm = findTimer(id, (timerCmd **) &prev);
+        if (tm == NULL) {
+        	logger("DBG", "Can not find timer to delete!");
         	return ;
         }    
         if (tm == timers) { //remove in head
@@ -240,7 +267,7 @@ void cancelTimer(char *id)
                         return;
                 }
         }       
-         
+	
 	tm->timer = NULL;
         free(tm);
 
@@ -265,12 +292,20 @@ extern void macroCmd(char *macro, cmdParams* params);
 
 void verifyTimers(int ticksInSec) 
 {
-	timerCmd *tm   = timers;
+        //logger("DBG", "verifyTimers()");
+	
+	timerCmd *tm = timers;
 
         while (tm != NULL) {
+		timerCmd *tm_next = tm->next;
+		
                 if (tm->status == TIMER_RUN) {	// do not process paused timers
+		        //DEBUG2("verifyTimers(): timer descr %s ", tm->timer->descr);
+		        //DEBUG2("verifyTimers(): timer ticks %d ", tm->ticks);
+		        //DEBUG2("verifyTimers(): timer tmout %d ", tm->timer->tdata->timeout);
+			
                 	if (tm->ticks >= (tm->timer->tdata->timeout * ticksInSec)) {
-                		DEBUG2("verifyTimers(): it is time to execute >%s<", (tm->timer->exec != NULL ? tm->timer->exec : "empty command"));
+                		DEBUG2("verifyTimers(): it is time to execute %s", tm->timer->descr);
 
 				macroCmd(tm->timer->descr,NULL);
                 
@@ -280,21 +315,16 @@ void verifyTimers(int ticksInSec)
                         		tm->times++;
                                 	if (tm->times >= tm->timer->tdata->times) {    // Cancel this timer 
                                 		DEBUG2("verifyTimers(): timer %s self-canceled", tm->timer->descr);
-                                
-                                        	// Cancel this timer
-                                        	timerCmd *tmc = tm;
-                                        	tm = tm->next;
-
-                                        	cancelTimer(tmc->timer->descr);
-                                
-                                        	continue;
-                                	}
+                                          	cancelTimer(tm->timer->descr);
+                                 	}
                         	}
                 	} else {
                 		tm->ticks++;
                 	}
-                }
-                tm   = tm->next;
+                } else if (tm->status == TIMER_CANCEL) { // delete canceled timers
+			deleteTimer(tm->timer->descr);
+		}
+                tm = tm_next;
         }
 }
 
@@ -407,7 +437,6 @@ void addInternalVars()
 	#else
         addVar("Xtest", "no", 2);	
 	#endif
-
 	
 	#ifdef USE_BLUEZ
         addVar("Bluez", "yes", 3);	
@@ -425,10 +454,10 @@ void addInternalVars()
         addVar("MixerChannel", "Master", 6);
 	
 	// Former "parameters"	
-	addVar("AutoConnect",  "false",  5);	
-        addVar("AutoRepeat",   "false",  5);
+	addVar(AUTOCONN_STR,   "false",  5);	
+        addVar(AUTOREPEAT_STR, "false",  5);
 	addVar("Baudrate",     "19200",  5);
-	addVar("CharSet",      "8859-1", 6);			// AT mode only
+	addVar(CHARSET_STR,    "8859-1", 6);			// AT mode only
 	addVar("CmerOff",      "",       0);	   		// AT mode only
 	addVar("CmerOn",       "",       0);			// AT mode only
 	addVar("Device",       "bluetooth:19", 12);
@@ -437,6 +466,8 @@ void addInternalVars()
 	addVar("ServiceName",  "anyRemote",    10);
 	addVar("TwoWayComm",   "false",  5);			// Send something to device ?
 	addVar("ToMainMenu",   "E",      1);			// AnyMenu  -> MainMenu - AT mode only
+	
+	addVar("IViewer",      "false",  5);			// Use iViewer tricks ?
 }
 
 void printVars() 
@@ -490,6 +521,11 @@ int setVar(const char *name, const char *val, int sz)
 		v->value = NULL;
 	}
         v->size = sz;
+	
+	// clean auto-repeat flag in any case
+	if (strcmp(name,AUTOREPEAT_STR) == 0) {
+		setRepeatNow(NULL);
+	}
 
         return EXIT_OK;
 }
